﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Songhay.Tests
{
    using Songhay.Extensions;

    /// <summary>
    /// General research tests.
    /// </summary>
    [TestClass]
    public class FrameworkFileTest
    {
        /// <summary>
        /// Initializes the test.
        /// </summary>
        [TestInitialize]
        public void InitializeTest()
        {
            #region remove previous test results:

            var directory = Directory.GetParent(TestContext.TestDir);

            directory.GetFiles()
                .OrderByDescending(f => f.LastAccessTime).Skip(1)
                .ForEachInEnumerable(f => f.Delete());

            directory.GetDirectories()
                .OrderByDescending(d => d.LastAccessTime).Skip(1)
                .ForEachInEnumerable(d => d.Delete(true));

            #endregion
        }

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        /// <summary>
        /// Framework file test: should find path of given length.
        /// </summary>
        [TestMethod]
        [Description("should find path of given length.")]
        public void ShouldFindPathOfGivenLength()
        {
            var directory = new DirectoryInfo(@"E:\~shares\dataRoot\Visual Studio 2010\Projects (Public)\");
            var dict = new Dictionary<string, int>();
            AddPathsToList(directory, dict);
            dict.ForEachInEnumerable(d =>
                {
                    if(d.Value > 200)
                        TestContext
                            .WriteLine(string.Format("{0}:{1}", d.Key, d.Value.ToString()));
                });
        }

        void AddPathsToList(DirectoryInfo info, Dictionary<string, int> dict)
        {
            info.GetDirectories()
                .ForEachInEnumerable(d =>
                {
                    dict.Add(d.FullName, d.FullName.Length);
                    d.GetDirectories()
                        .ForEachInEnumerable(d2 =>
                        {
                            AddPathsToList(d2, dict);
                        });
                });
        }

        [DeploymentItem("FrameworkFileTest-ShouldSortTextFileData.txt")]
        [TestMethod]
        public void ShouldSortTextFileData()
        {
            var output = new List<string>();

            var path = string.Format(@"{0}\FrameworkFileTest-ShouldSortTextFileData.txt", TestContext.DeploymentDirectory);

            var stream = new FileStream(path, FileMode.Open);
            try
            {
                using(var sr = new StreamReader(stream))
                {
                    var dataArray = sr.ReadToEnd().Replace(Environment.NewLine, ",").Split(',');
                    dataArray.Select(s => new { Car = s, Count = dataArray.Where(i => i == s).Count() })
                        .OrderByDescending(o => o.Count).ThenBy(o => o.Car)
                        .GroupBy(o => o.Car)
                        .ForEachInEnumerable(o =>
                    {
                        TestContext.WriteLine(o.Key + " = " + o.Count().ToString());
                    });
                }
            }
            finally
            {
                if(stream != null) stream.Dispose();
            }
        }

        /// <summary>
        /// Framework file test: should write text file with <see cref="System.IO.StreamWriter"/>.
        /// </summary>
        [TestMethod]
        [Description("Should write text file with StreamWriter.")]
        public void ShouldWriteTextFileWithStreamWriter()
        {
            var fileText = @"
This is the text to write to the test file.
This sentence should be on a line all by iteself.
According to Notepad++ this file should be in ANSI format by default.
According to Notepad++, when Encoding.UTF8 is specified the encoding is UTF8.
This is the end of the file.
            ";
            var path = ConfigurationManager.AppSettings.Get("TESTPROJECTFOLDER") + "FrameworkFileTest-ShouldWriteTextFileWithStreamWriter.txt";

            var stream = new FileStream(path, FileMode.Create);
            try
            {
                using(var sw = new StreamWriter(stream, Encoding.UTF8))
                {
                    sw.Write(fileText);
                }
            }
            finally
            {
                if(stream != null) stream.Dispose();
            }
        }

        /// <summary>
        /// Framework file test: should write text file with <see cref="System.Xml.XmlTextWriter"/>.
        /// </summary>
        [TestMethod]
        [Description("Should write text file with XmlTextWriter.")]
        public void ShouldWriteTextFileWithXmlTextWriter()
        {
            var path = ConfigurationManager.AppSettings.Get("TESTPROJECTFOLDER") + "FrameworkFileTest-ShouldWriteTextFileWithXmlTextWriter.xml";
            var stream = new FileStream(path, FileMode.Create);
            try
            {
                using(var writer = new XmlTextWriter(stream, Encoding.UTF8))
                {
                    writer.WriteStartElement("root");
                    writer.WriteAttributeString("xmlns", "x", null, "urn:1");
                    writer.WriteStartElement("item", "urn:1");
                    writer.WriteString("This is the text to write to the test file.");
                    writer.WriteEndElement();
                    writer.WriteStartElement("item", "urn:1");
                    writer.WriteString("According to Notepad++, when Encoding.UTF8 is specified the encoding is UTF8.");
                    writer.WriteEndElement();
                    writer.WriteEndElement();
                }
            }
            finally
            {
                if(stream != null) stream.Dispose();
            }
        }

        /// <summary>
        /// Framework file test: should write to <see cref="System.Environment.SpecialFolder.MyDocuments"/> folder.
        /// </summary>
        [TestMethod]
        [Description("Should write directory in MyDocuments folder.")]
        public void ShouldWriteToMyDocumentsFolder()
        {
            var path = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) +
                @"\WordWalkingStick\";
            if(!Directory.Exists(path)) Directory.CreateDirectory(path);
        }
    }
}
