﻿using System.Collections.ObjectModel;
using System.ComponentModel.DataAnnotations;

namespace Songhay.Wpf.BiggestBox.IndexPart.ViewModels
{
    using Songhay.Wpf.Mvvm.ViewModels;

    /// <summary>
    /// Models Validation Sample
    /// </summary>
    public class ValidationSampleViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance
        /// of the <see cref="ValidationSampleViewModel"/> class.
        /// </summary>
        public ValidationSampleViewModel()
        {
            this.SetSubclassForValidation(this.GetType());

            this.SelectedUSStates =
                new ObservableCollection<string>
                {
                    {" "},
                    {"[Other]"},
                    {"[Refuse to Answer]"},
                    {"Arkansas"},
                    {"California"},
                    {"Louisiana"},
                    {"Texas"}
                };

            this.YesNoValues =
                new ObservableCollection<string>
                {
                    {"Yes"},
                    {"No"}
                };
        }

        /// <summary>
        /// Gets or sets the email.
        /// </summary>
        /// <value>The email.</value>
        [Required(ErrorMessage = "Email is required.")]
        [RegularExpression(@"(?i:^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$)")]
        public string Email
        {
            get
            {
                return this._email;
            }
            set
            {
                if((!string.IsNullOrEmpty(value)) &&
                    (this._email == value)) return;

                this._email = value;
                base.RaisePropertyChanged("Email");
            }
        }

        /// <summary>
        /// Gets or sets the first name.
        /// </summary>
        /// <value>The first name.</value>
        [Required]
        public string FirstName
        {
            get
            {
                return this._firstName;
            }
            set
            {
                if(!string.IsNullOrEmpty(value) &&
                    (this._firstName == value)) return;

                this._firstName = value;
                base.RaisePropertyChanged("FirstName");
            }
        }

        /// <summary>
        /// Gets or sets a value indicating
        /// whether this instance is residing in US.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance
        /// 	is residing in US; otherwise, <c>false</c>.
        /// </value>
        public bool IsResidingInUS
        {
            get
            {
                return this._isResidingInUS;
            }
            set
            {
                if(!this.ValidateIsResidingInUS(value)) return;
                this._isResidingInUS = value;
                base.RaisePropertyChanged("IsResidingInUS");
            }
        }

        /// <summary>
        /// Gets or sets the last name.
        /// </summary>
        /// <value>The last name.</value>
        [Required]
        public string LastName
        {
            get
            {
                return this._lastName;
            }
            set
            {
                if(!string.IsNullOrEmpty(value) && (this._lastName == value)) return;

                _lastName = value;
                base.RaisePropertyChanged("LastName");
            }
        }

        /// <summary>
        /// Gets or sets the selected US state.
        /// </summary>
        /// <value>The state of the selected US state.</value>
        [Required]
        public string SelectedUSState
        {
            get
            {
                return this._selectedUSState;
            }
            set
            {
                if(!string.IsNullOrEmpty(value) &&
                    (this._selectedUSState == value)) return;

                if(!string.IsNullOrEmpty(value)) value = value.Trim();
                this._selectedUSState = value;
                base.RaisePropertyChanged("SelectedUSState");
            }
        }

        /// <summary>
        /// Gets or sets the selected US states.
        /// </summary>
        /// <value>The selected US states.</value>
        public ObservableCollection<string> SelectedUSStates { get; set; }

        /// <summary>
        /// Gets or sets the yes-no collection.
        /// </summary>
        /// <value>The yes-no collection.</value>
        public ObservableCollection<string> YesNoValues { get; set; }

        bool ValidateIsResidingInUS(bool isResidingInUS)
        {
            var isValid = (isResidingInUS && (this._selectedUSState != " ")) ||
                (this._selectedUSState == "[Refuse to Answer]");

            /*if(!isValid)
            {
                base.AddError(this, "IsResidingInUS", ISRESIDINGINUS_ERROR, false);
            }
            else
            {
                base.RemoveError(this, "IsResidingInUS", ISRESIDINGINUS_ERROR);
            }*/

            return isValid;
        }

        string _email;
        string _firstName;
        bool _isResidingInUS;
        string _lastName;
        string _selectedUSState;

        const string ISRESIDINGINUS_ERROR =
            "You have not selected a U.S. state (or [Other]). " +
            "Are you sure you do NOT reside in the U.S?";

    }
}
