﻿using System.Windows;
using System.Windows.Controls;

namespace Songhay.Wpf.BiggestBox.IndexPart.Views.CustomControls
{
    /// <summary>
    /// Follow steps 1a or 1b and then 2 to use this custom control in a XAML file.
    ///
    /// Step 1a) Using this custom control in a XAML file that exists in the current project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:Songhay.Wpf.BiggestBox.Views.UserControls"
    ///
    ///
    /// Step 1b) Using this custom control in a XAML file that exists in a different project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:Songhay.Wpf.BiggestBox.Views.UserControls;assembly=Songhay.Wpf.BiggestBox.Views.UserControls"
    ///
    /// You will also need to add a project reference from the project where the XAML file lives
    /// to this project and Rebuild to avoid compilation errors:
    ///
    ///     Right click on the target project in the Solution Explorer and
    ///     "Add Reference"->"Projects"->[Browse to and select this project]
    ///
    ///
    /// Step 2)
    /// Go ahead and use your control in the XAML file.
    ///
    ///     <MyNamespace:DynamicGrid/>
    ///
    /// </summary>
    public class DynamicGrid : Grid
    {
        /// <summary>
        /// Initializes the <see cref="DynamicGrid"/> class.
        /// </summary>
        static DynamicGrid()
        {
            FrameworkElement.DefaultStyleKeyProperty
                .OverrideMetadata(typeof(DynamicGrid),
                    new FrameworkPropertyMetadata(typeof(DynamicGrid)));
        }

        /// <summary>
        /// Measures the children of a <see cref="T:System.Windows.Controls.Grid"/> in anticipation of arranging them during the <see cref="M:System.Windows.Controls.Grid.ArrangeOverride(System.Windows.Size)"/> pass.
        /// </summary>
        /// <param name="constraint">Indicates an upper limit size that should not be exceeded.</param>
        /// <returns>
        /// 	<see cref="T:System.Windows.Size"/> that represents the required size to arrange child content.
        /// </returns>
        protected override Size MeasureOverride(Size constraint)
        {
            if(this._updateChildenIndices || this._columnCount != base.ColumnDefinitions.Count)
            {
                this._updateChildenIndices = false;
                this._columnCount = base.ColumnDefinitions.Count;

                //  Update the number of rows
                int newRowCount = ((base.Children.Count - 1) / this._columnCount + 1);
                while(base.RowDefinitions.Count < newRowCount)
                {
                    base.RowDefinitions.Add(new RowDefinition());
                }
                if(base.RowDefinitions.Count > newRowCount)
                {
                    base.RowDefinitions.RemoveRange(newRowCount, base.RowDefinitions.Count - newRowCount);
                }

                //  Update children indices
                for(int i = 0, childrenCount = base.Children.Count; i < childrenCount; ++i)
                {
                    UIElement child = base.Children[i];
                    Grid.SetColumn(child, i % _columnCount);
                    Grid.SetRow(child, i / _columnCount);
                }
            }

            return (base.MeasureOverride(constraint));
        }

        /// <summary>
        /// Called when the visual children of a <see cref="T:System.Windows.Controls.Grid"/> element change.
        /// </summary>
        /// <param name="visualAdded">Identifies the visual child that's added.</param>
        /// <param name="visualRemoved">Identifies the visual child that's removed.</param>
        protected override void OnVisualChildrenChanged(DependencyObject visualAdded, DependencyObject visualRemoved)
        {
            this._updateChildenIndices = true;
            base.OnVisualChildrenChanged(visualAdded, visualRemoved);
        }

        bool _updateChildenIndices = true; // A value of "true" forces children to be re-indexed at the next opportunity
        int _columnCount;
    }
}
