﻿using System;
using System.Globalization;
using System.Text.RegularExpressions;

namespace Songhay.Xml
{
    /// <summary>
    /// Static members for HTML text processing.
    /// </summary>
    public static class HtmlUtility
    {
        /// <summary>
        /// Returns a string of marked up text compatible
        /// with browsers that do not support XHTML
        /// (loosely towards HTML 4.x W3C standard).
        /// </summary>
        /// <param name="inputValue">
        /// A string of marked up text.
        /// </param>
        public static string ConvertToHtml(string inputValue)
        {
            if (string.IsNullOrEmpty(inputValue)) return null;

            //Minimize selected XHTML block elements.
            inputValue
                = Regex.Replace(inputValue, @"</(base|isindex|link|meta)>",
                string.Empty, RegexOptions.IgnoreCase);

            //Remove XHTML html element attributes.
            inputValue
                = Regex.Replace(inputValue, @"<html*>",
                "<html>", RegexOptions.IgnoreCase);

            //Remove XHTML element minimization.
            inputValue = Regex.Replace(inputValue, @"\s*/>", ">");

            //Remove XHTML attribute minimization.
            foreach (Match mTag in Regex.Matches(inputValue, @"<[^/][^>]*>"))
            {
                //An opening inputValue element has been found.
                string strReplace = mTag.Value;
                foreach (Match mAttr in Regex.Matches(strReplace, @"\s+(.+)\s*=\s*""\1"""))
                {
                    //XHTML minimization found (e.g. foo="foo").
                    strReplace
                        = strReplace.Replace(mAttr.Value,
                        string.Concat(" ", mAttr.Groups[1].Value));
                }
                inputValue = inputValue.Replace(mTag.Value, strReplace);
            }

            return inputValue;
        }

        /// <summary>
        /// Attempts to convert HTML to well-formed XML.
        /// </summary>
        /// <param name="html">
        /// An HTML 
        /// <see cref="System.String"/>.
        /// </param>
        /// <returns><see cref="System.String"/></returns>
        /// <remarks>This task is simpler than converting to XHTML.</remarks>
        public static string ConvertToXml(string html)
        {
            if (string.IsNullOrEmpty(html)) return null;

            Regex re;
            MatchEvaluator me;

            //Remove xmlns attributes:
            html = Regex.Replace(html, @"\s*xmlns\s*=\s*""[^""]+""\s*", string.Empty);

            //Close open elements:
            me = new MatchEvaluator(EvaluateOpenElement);

            re = new Regex(@"<\s*(br|hr|img|link|meta)([^>]*)(>)",
                RegexOptions.IgnoreCase);
            html = re.Replace(html, me);

            //Find attributes without quotes:
            me = new MatchEvaluator(EvaluateBadAttribute);

            re = new Regex(@"<\s*[^>]+\s((\w+)\s*=\s*(\w+))\s*[^>]*\/*>",
                RegexOptions.IgnoreCase);
            html = re.Replace(html, me);

            //Generate attributes:
            me = new MatchEvaluator(EvaluateAttribute);

            re = new Regex(@"<\s*[^>]+\s(checked|nobreak|nosave|selected)[^=>]*\/*>",
                RegexOptions.IgnoreCase);
            html = re.Replace(html, me);

            //Look for Query strings with raw ampersands:
            foreach (Match m in Regex.Matches(html, @"href\s*=\s*""[^""]+"""))
            {
                if (!m.Value.Contains("&amp;"))
                    html = html.Replace(m.Value, m.Value.Replace("&", "&amp;"));
            }

            //Replace the CDATA "xmlns" with "x…mlns" (adds a soft-hyphen):
            html = html.Replace("xmlns", "x…mlns");

            return html;
        }

        /// <summary>
        /// Returns an XHTML string derived from a .NET procedure.
        /// </summary>
        /// <param name="xmlFragment">
        /// A well-formed <see cref="System.String"/> of XML.
        /// </param>
        /// <remarks>
        /// This member addresses certain quirks
        /// that well-formed XML cannot have in a contemporary Web browser.
        /// </remarks>
        public static string FormatXhtmlElements(string xmlFragment)
        {
            if (string.IsNullOrEmpty(xmlFragment)) return null;

            //Maximize selected empty minimized block elements.
            string newValue;
            foreach (Match m in Regex.Matches(xmlFragment, @"<(a|iframe|td|th|script)\s+[^>]*\s*(\/>)", RegexOptions.IgnoreCase))
            {
                if (m.Groups.Count == 2)
                {
                    newValue = m.Value.Replace(m.Groups[1].Value,
                        string.Concat("></", m.Groups[0].Value, ">"));
                    xmlFragment = xmlFragment.Replace(m.Value, newValue);
                }
            }

            return xmlFragment;
        }

        /// <summary>
        /// Returns the …inner… fragment of XML
        /// from the specified unique element.
        /// </summary>
        /// <param name="xmlFragment">
        /// A well-formed <see cref="System.String"/> of XML.
        /// </param>
        /// <param name="elementName">
        /// The local name of the element in the XML string.
        /// </param>
        public static string GetInnerXml(string xmlFragment, string elementName)
        {
            if (string.IsNullOrEmpty(xmlFragment)) return null;

            string ret = xmlFragment;

            string pattern = string.Format(CultureInfo.InvariantCulture, @"<{0}[^>]*>((\s*.+\s*)+)<\/{0}>", elementName);
            foreach (Match m in Regex.Matches(ret, pattern, RegexOptions.IgnoreCase))
            {
                if (m.Groups.Count > 1) ret = m.Groups[1].Value;
                break;
            }

            //Remove first four spaces at start of line.
            ret = Regex.Replace(ret, @"\r\n\W{4}", "\r\n");

            return ret;
        }

        /// <summary>
        /// Emits a public <c>DOCTYPE</c> tag.
        /// </summary>
        /// <returns>
        /// A public <c>DOCTYPE</c> tag for XHTML 1.0 transitional.
        /// </returns>
        public static string PublicDocType()
        {
            return PublicDocType("html",
                "-//W3C//DTD XHTML 1.0 Transitional//EN",
                    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd");
        }

        /// <summary>
        /// Emits a public <c>DOCTYPE</c> tag.
        /// </summary>
        /// <param name="rootElement">
        /// The root element of the DTD.
        /// </param>
        /// <param name="publicIdentifier">
        /// The public identifier of the DTD.
        /// </param>
        /// <param name="resourceReference">
        /// The link to reference material of the DTD.
        /// </param>
        /// <returns>
        /// A public <c>DOCTYPE</c> tag.
        /// </returns>
        public static string PublicDocType(string rootElement,
            string publicIdentifier, string resourceReference)
        {
            return string.Format(CultureInfo.InvariantCulture, "<!DOCTYPE {0} PUBLIC \"{1}\" \"{2}\">",
                rootElement, publicIdentifier, resourceReference);
        }

        #region Regular Expression Match Evaluators

        private static string EvaluateAttribute(Match m)
        {
            string ret = m.Value;

            if (m.Groups.Count == 2)
            {
                ret =
                    m.Groups[0].Value.Replace(m.Groups[1].Value,
                        string.Format(CultureInfo.InvariantCulture, @"{0}=""{0}""", m.Groups[1].Value));
            }

            return ret;
        }

        private static string EvaluateBadAttribute(Match m)
        {
            string ret = m.Value;

            if (m.Groups.Count == 4)
            {
                ret =
                    m.Groups[0].Value.Replace(m.Groups[1].Value,
                        string.Format(CultureInfo.InvariantCulture, "{0}=\"{1}\"", m.Groups[2].Value, m.Groups[3].Value));
            }

            return ret;
        }

        private static string EvaluateOpenElement(Match m)
        {
            string ret = m.Value;

            if (m.Groups.Count == 4)
            {
                //Refuse closed elements:
                if (m.Groups[2].Value.Trim().EndsWith("/", StringComparison.OrdinalIgnoreCase)) return ret;

                string oldValue = m.Groups[3].Value;

                if (oldValue.IndexOf(">", StringComparison.OrdinalIgnoreCase) != -1)
                    ret = ret.Replace(oldValue, " />");
            }
            return ret;
        }

        #endregion
    }
}
