﻿using System;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Songhay.Net
{
    /// <summary>
    /// Static members for <see cref="System.Net.WebRequest"/>.
    /// </summary>
    public static partial class HttpVerbAsyncUtility
    {
        /// <summary>
        /// Gets as task.
        /// </summary>
        /// <param name="resourceIndicator">The resource indicator.</param>
        public static Task<string> GetAsync(Uri resourceIndicator)
        {
            return GetAsync(resourceIndicator, null);
        }

        /// <summary>
        /// Gets as task.
        /// </summary>
        /// <param name="resourceIndicator">The resource indicator.</param>
        /// <param name="getCompleteAction">The get complete action.</param>
        /// <remarks>
        /// For details, see the “Converting an Event-Based Pattern” section in 
        /// “Simplify Asynchronous Programming with Tasks”
        /// by Igor Ostrovsky
        /// [http://msdn.microsoft.com/en-us/magazine/ff959203.aspx]
        /// </remarks>
        public static Task<string> GetAsync(Uri resourceIndicator, Action<object, DownloadStringCompletedEventArgs> getCompleteAction)
        {
            var completionSource = new TaskCompletionSource<string>();
            var web = new WebClient();
            web.Encoding = Encoding.UTF8;

            web.DownloadStringCompleted += (o, args) =>
            {
                try
                {
                    if(args.Error != null) completionSource.SetException(args.Error);
                    else if(args.Cancelled) completionSource.SetCanceled();
                    else
                    {
                        completionSource.SetResult(args.Result);
                        if(getCompleteAction != null) getCompleteAction.Invoke(o, args);
                    }
                }
                finally
                {
#if !SILVERLIGHT
                    web.Dispose();
#endif
                }
            };

            web.DownloadStringAsync(resourceIndicator);

            return completionSource.Task;
        }
    }
}
